﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Windows.Forms;
using HeuristicLab.Common;
using HeuristicLab.Problems.DataAnalysis;
using HeuristicLab.Problems.Instances.DataAnalysis.Regression.Matlab.Api;
using HeuristicLab.Problems.Instances.DataAnalysis.Regression.Matlab.Api.Types;

namespace HeuristicLab.Problems.Instances.DataAnalysis.Views.Regression.Matlab {
  public partial class RegressionMatlabImportDialog {
    private abstract class MLVariablesLoader : IMLVariablesLoader {

      protected IList<MLVariableInfo> _selectedVariables;

      protected MLDatatype _mlDatatype;

      protected CheckedListBox _variablesListBox;

      public event EventHandler<EventArgs<int>> NumberOfSelectedVariablesChanged;

      public IEnumerable<MLVariableInfo> SelectedVariables { get { return _selectedVariables; } }
            
      public MLVariablesLoader(RegressionMatlabImportDialog importDialog, CheckedListBox variablesListBox, MLDatatype mlDatatype) {
        _variablesListBox = variablesListBox;
        _mlDatatype = mlDatatype;
        _selectedVariables = new List<MLVariableInfo>();

        _variablesListBox.ItemCheck += (s, e) => importDialog.BeginInvoke((MethodInvoker)(() => CheckedItemsChanged(s, e)));
      }
      
      #region Events
      private void CheckedItemsChanged(object s, object e) {
        _selectedVariables.Clear();
        foreach (MLVariableInfo item in _variablesListBox.CheckedItems) {
          if (item.Datatype == _mlDatatype) {
            _selectedVariables.Add(item);
          }
        }
        NumberOfSelectedVariablesChanged?.Invoke(this, new EventArgs<int>(_selectedVariables.Count));
      }
      #endregion

      public abstract Dataset GetPreviewDataset(IMatlabConnector mlConnector, CancellationToken token);

      public void DisplayVariables(IEnumerable<MLVariableInfo> variables) {
        foreach (var item in variables) {
          if (item.Datatype == _mlDatatype) {
            _variablesListBox.Items.Add(item, false);
          }
        }
      }
    }
  }
}
